/*
 * deviced
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <unistd.h>
#include <bundle.h>
#include <eventsystem.h>
#include <libsyscommon/dbus-system.h>
#include <libsyscommon/dbus-systemd.h>
#include <libsyscommon/systemd-state.h>

#include "core/log.h"
#include "core/device-notifier.h"
#include "core/common.h"
#include "display/poll.h"
#include "display/display-ops.h"

#define SYSTEMD_DBUS_SIGNAL_SYSTEM_STARTUP_FINISHED     "StartupFinished"
#define SYSTEMD_DBUS_SIGNAL_USER_STARTUP_FINISHED       "UserSessionStartupFinished"

static guint sig_id[2] = {0, 0};

void remove_booting_done_handler(void *data)
{
	unsubscribe_dbus_signal(NULL, sig_id[0]);
	unsubscribe_dbus_signal(NULL, sig_id[1]);
}

static void booting_done_received(GDBusConnection  *conn,
	const gchar      *sender,
	const gchar      *path,
	const gchar      *iface,
	const gchar      *name,
	GVariant         *param,
	gpointer          data)
{
	static int system_done = 0;
	static int user_done = 0;

	if (strcmp(name, SYSTEMD_DBUS_SIGNAL_SYSTEM_STARTUP_FINISHED) == 0) {
		if (system_done)
			return;

		system_done = check_system_boot_finished();
		if (system_done == 0) {
			_E("System session is not ready yet.");
			return;
		}
		_I("System session is ready.");
		device_notify_once(DEVICE_NOTIFIER_BOOTING_DONE, &system_done);

	} else if (strcmp(name, SYSTEMD_DBUS_SIGNAL_USER_STARTUP_FINISHED) == 0) {
		if (user_done)
			return;
		user_done = 1;
		_I("User session is ready.");
	}

	if (!system_done || !user_done)
		return;

	remove_booting_done_handler(NULL);

	_I("Real booting done. Unlock LCD_OFF.");
	if (disp_plgn.pm_unlock_internal)
		disp_plgn.pm_unlock_internal(INTERNAL_LOCK_BOOTING, LCD_OFF, PM_SLEEP_MARGIN);

	_I("Signal booting done.");
}

void add_booting_done_handler(void *data)
{
	/* System Session is loaded completely */
	/*ret = */
	sig_id[0] = subscribe_dbus_signal(NULL,
					SYSTEMD_DBUS_PATH,
					SYSTEMD_DBUS_IFACE_MANAGER,
					SYSTEMD_DBUS_SIGNAL_SYSTEM_STARTUP_FINISHED,
					booting_done_received,
					NULL, NULL);

	if (sig_id[0] <= 0)
		_E("Failed to init dbus signal(%s).", SYSTEMD_DBUS_SIGNAL_SYSTEM_STARTUP_FINISHED);

	/* User Session is loaded completely */
	sig_id[1] = subscribe_dbus_signal(NULL,
					SYSTEMD_DBUS_PATH,
					SYSTEMD_DBUS_IFACE_MANAGER,
					SYSTEMD_DBUS_SIGNAL_USER_STARTUP_FINISHED,
					booting_done_received,
					NULL, NULL);

	if (sig_id[1] <= 0)
		_E("Failed to init dbus signal(%s).", SYSTEMD_DBUS_SIGNAL_USER_STARTUP_FINISHED);
}
